from base64 import urlsafe_b64encode
from datetime import datetime
from json import dumps
from socket import error
from typing import Dict
from urllib.error import URLError
from urllib.parse import urljoin

from mocket import mocketize
from mocket.mockhttp import Entry
from pytest import mark, raises

from .auth import Auth, is_expired_token
from .exc import AuthError


def _encode(claims: Dict[str, any]) -> str:
    encoded = urlsafe_b64encode(dumps(claims).encode("utf-8")).decode("utf-8")
    return f"header.{encoded}.sign"


@mark.parametrize(
    "claims, is_expired",
    (
        (None, False),
        ({}, False),
        ({"exp": datetime.now().timestamp() + 10}, False),
        ({"exp": datetime.now().timestamp() - 10}, True),
    ),
)
def test_jwt_expired(claims: Dict[str, any], is_expired):
    token = _encode(claims)
    assert is_expired_token(token) == is_expired


@mocketize(strict_mode=True)
@mark.parametrize(
    "mocket_args, exception",
    (({"status": 403}, AuthError), ({"exception": error()}, URLError)),
)
def test_auth_token_failure(memory_cache, mocket_args: Dict, exception):
    auth = Auth(client_id="test", client_secret="test", cache=memory_cache)

    Entry.single_register("POST", urljoin(auth.auth_url, "oauth/token"), **mocket_args)

    with raises(exception):
        auth.token()


@mocketize(strict_mode=True)
def test_auth_token(memory_cache):
    auth = Auth(client_id="test", client_secret="test", cache=memory_cache)

    Entry.single_register(
        "POST",
        urljoin(auth.auth_url, "oauth/token"),
        body=dumps({"access_token": "test-token"}),
    )

    assert auth.token() == "test-token"



from datetime import datetime
from typing import Any, Optional

from .base import AbstractApi


class ArchiveV1(AbstractApi):
    """
    Client interface to the Archive API located at
    https://api.satellitevu.com/archive/v1/docs.
    """

    api_path = "archive/v1"
    scopes = ["stac:r"]

    def search(
        self,
        *,
        intersects: Optional[Any] = None,
        date_from: Optional[datetime] = None,
        date_to: Optional[datetime] = None,
        limit=10,
        page_token: Optional[str] = None,
        **kwargs,
    ):
        """
        Perform a search on Satellite Vu's STAC. Relevant documentation is located
        at https://api.satellitevu.com/archive/v1/docs#operation/Search_search_post

        Kwargs:
            intersects: Optional dictionary with keys "coordinates" and "geometry"
            type that search results intersect with. Available geometry types include:
            "Point","MultiPoint", "LineString", "MultiLineString", "Polygon",
            "MultiPolygon". For example:
            intersects = {"coordinates":[-1.065151, 51.163899], "type" : "Point"}.

            date_from: Optional datetime representing the start date of the search.

            date_to: Optional datetime representing the end date of the search.

            limit:  Number of search results (defaults to 10) to be returned.

            bbox: List of points (min longitude, min latitude, max longitude,
            max latitude) specifying a bounding box to search within.

            ids: List of strings specifying the desired STAC identifiers
            e.g. ["20221010T222611000_basic_0_TABI"].

            collections: List of strings specifying the collections within which
            to search. Values include "basic" and "relative".

            sortby: List of parameters specifying the field and direction the results
            are sorted by e.g. [{"field": "datetime", "direction": "desc"}].

        Returns:
            An instance of ResponseWrapper where the .json() method returns a
            FeatureCollection (dictionary containing keys: "type", "features",
            "link", "context") of the catalogue items fulfilled by the search
            criteria.

        """
        url = self.url("/search")
        payload = {
            **kwargs,
            "intersects": intersects,
            "limit": limit,
            "token": page_token,
            "datetime": "/".join(
                filter(
                    lambda x: x,
                    [
                        date_from and date_from.isoformat(),
                        date_to and date_to.isoformat(),
                    ],
                )
            ),
        }

        return self.make_request(
            method="POST", url=url, json={k: v for k, v in payload.items() if v}
        )



from base64 import b64decode
from datetime import datetime
from hashlib import sha1
from json import loads
from logging import getLogger
from typing import Optional
from urllib.parse import urljoin

from satellitevu.config import AUDIENCE, AUTH_URL
from satellitevu.http import AbstractClient, UrllibClient

from .cache import AbstractCache, AppDirCache
from .exc import AuthError

logger = getLogger(__file__)


def is_expired_token(token: str) -> bool:
    json = b64decode(token.split(".")[1] + "==")
    claims = loads(json)
    if not claims or "exp" not in claims:
        return False
    exp = float(claims["exp"])
    exp_dt = datetime.fromtimestamp(exp)
    return exp_dt <= datetime.now()


class Auth:
    client_id: str
    client_secret: str
    audience: str

    cache: AbstractCache

    auth_url: str
    client: AbstractClient

    def __init__(
        self,
        *,
        client_id: str,
        client_secret: str,
        audience: Optional[str] = None,
        cache: Optional[AbstractCache] = None,
        auth_url: Optional[str] = None,
        client: Optional[AbstractClient] = None,
    ):
        self.client_id = client_id
        self.client_secret = client_secret
        self.audience = audience or AUDIENCE

        self.cache = cache or AppDirCache()
        self.auth_url = auth_url or AUTH_URL
        self.client = client or UrllibClient()

    def token(self, scopes=[]) -> str:
        cache_key = sha1(self.client_id.encode("utf-8"))
        cache_key.update("".join(scopes).encode("utf-8"))

        token = self.cache.load(cache_key.hexdigest())

        if not token or is_expired_token(token):
            token = self._auth(scopes)
            self.cache.save(cache_key.hexdigest(), token)

        return token

    def _auth(self, scopes=[]) -> str:
        logger.info("Performing client_credential authentication")
        token_url = urljoin(self.auth_url, "oauth/token")
        response = self.client.post(
            token_url,
            headers={"content-type": "application/x-www-form-urlencoded"},
            data={
                "grant_type": "client_credentials",
                "client_id": self.client_id,
                "client_secret": self.client_secret,
                "audience": self.audience,
                "scope": " ".join(scopes),
            },
        )

        if response.status != 200:
            raise AuthError(
                "Unexpected error code for client_credential flow: "
                f"{response.status} - {response.text}"
            )
        try:
            payload = response.json()
            return payload["access_token"]
        except Exception:
            raise AuthError(
                "Unexpected response body for client_credential flow: " + response.text
            )



from abc import ABC
from urllib.parse import urljoin

from satellitevu.auth.exc import Api401Error, Api403Error
from satellitevu.http import AbstractClient


class AbstractApi(ABC):
    client: AbstractClient
    base_url: str
    api_path: str
    scopes = []

    def __init__(self, client: AbstractClient, base_url: str):
        self.client = client
        self.base_url = base_url

    def url(self, path: str) -> str:
        api_base_url = urljoin(self.base_url, self.api_path.lstrip("/"))
        if api_base_url[-1] != "/":
            api_base_url += "/"
        return urljoin(api_base_url, path.lstrip("/"))

    def make_request(self, *args, **kwargs):
        if "scopes" not in kwargs:
            kwargs["scopes"] = self.scopes
        response = self.client.request(*args, **kwargs)

        if response.status == 401:
            raise Api401Error("Unauthorized to make this request.")
        elif response.status == 403:
            raise Api403Error(
                (
                    "Not permitted to perform this action. "
                    "Please contact Satellite Vu for assistance."
                )
            )

        return response



from datetime import datetime
from typing import Tuple, Union
from uuid import UUID

from .base import AbstractApi

# TODO: Tests


class OtmV1(AbstractApi):
    """
    Client interface to the OTM API located at
    https://api.satellitevu.com/otm/v1/docs.
    """

    api_path = "otm/v1"
    scopes = ["tasking:w"]

    def post_feasibility(
        self,
        *,
        coordinates: Union[Tuple[float, float], Tuple[float, float, float]],
        date_from: datetime,
        date_to: datetime,
    ):
        """
        TODO: Docs
        """
        url = self.url("/tasking/feasibilities/")
        payload = {
            "type": "Feature",
            "geometry": {
                "type": "Point",
                "coordinates": coordinates,
            },
            "properties": {
                "datetime": f"{date_from.isoformat()}/{date_to.isoformat()}"
            },
        }

        return self.make_request(
            method="POST", url=url, json={k: v for k, v in payload.items() if v}
        )

    def get_feasibility(self, *, id: Union[UUID, str]):
        """
        TODO: Docs
        """
        return self.make_request(
            method="GET",
            url=self.url(f"/tasking/feasibilities/{str(id)}"),
        )



import os
from abc import ABC, abstractmethod
from configparser import ConfigParser, DuplicateSectionError
from os import replace
from pathlib import Path
from tempfile import NamedTemporaryFile
from typing import Optional

from appdirs import user_cache_dir


class AbstractCache(ABC):
    """
    Abstract cache interface, implemented by actual cache classes
    """

    @abstractmethod
    def save(self, client_id: str, value: str):
        pass

    @abstractmethod
    def load(self, client_id: str) -> Optional[str]:
        pass


class MemoryCache(AbstractCache):
    _items = {}

    def save(self, client_id: str, value: str):
        self._items[client_id] = value

    def load(self, client_id: str) -> Optional[str]:
        return self._items.get(client_id)


class AppDirCache(AbstractCache):
    """
    File based token cache using an INI file in the user's cache dir or given dir.
    """

    cache_dir: Path
    cache_file: Path

    def __init__(self, cache_dir: Optional[str] = None):
        self.cache_dir = Path(cache_dir if cache_dir else user_cache_dir("SatelliteVu"))
        self.cache_file = self.cache_dir / "tokencache"

        if not os.path.exists(self.cache_dir):
            os.makedirs(self.cache_dir)

    def save(self, client_id: str, value: str):
        parser = ConfigParser()
        parser.read(self.cache_file)

        try:
            parser.add_section(client_id)
        except DuplicateSectionError:
            pass
        parser[client_id]["access_token"] = value

        with NamedTemporaryFile("w", dir=str(self.cache_dir), delete=False) as handle:
            parser.write(handle)
        replace(handle.name, self.cache_file)

    def load(self, client_id: str) -> Optional[str]:
        try:
            parser = ConfigParser()
            parser.read(self.cache_file)

            return parser[client_id]["access_token"]
        except (FileNotFoundError, KeyError):
            return None



from typing import Dict, Optional, Union
from warnings import warn

from satellitevu.apis.archive import ArchiveV1
from satellitevu.apis.orders import OrdersV1
from satellitevu.apis.otm import OtmV1
from satellitevu.auth import AbstractCache, Auth
from satellitevu.config import GATEWAY
from satellitevu.http import AbstractClient, UrllibClient


class FutureApis:
    otm: OtmV1

    _called: Dict[str, bool] = {}

    def __init__(self, client: AbstractClient, gateway_url: str):
        self.otm = OtmV1(client, gateway_url)

    def __getattribute__(self, __name: str):
        attr = super().__getattribute__(__name)
        if __name != "_called":
            has_been_called = self._called.get(__name, False)
            self._called[__name] = True
            if not has_been_called:
                warn(
                    f"{__name} is a not yet a stable API. Use at own risk",
                    FutureWarning,
                )
        return attr


class Client:
    _client: AbstractClient
    _gateway_url: str

    auth: Auth

    archive_v1: ArchiveV1
    orders_v1: OrdersV1

    future: FutureApis

    def __init__(
        self,
        client_id: str,
        client_secret: str,
        *,
        audience: Optional[str] = None,
        cache: Optional[AbstractCache] = None,
        auth_url: Optional[str] = None,
        http_client: Optional[AbstractClient] = None,
        gateway_url: Optional[str] = None,
    ):
        self._gateway_url = gateway_url or GATEWAY
        self._client = http_client or self._setup_client()

        self.auth = Auth(
            client_id=client_id,
            client_secret=client_secret,
            audience=audience,
            cache=cache,
            auth_url=auth_url,
            client=self._client,
        )
        self._client.set_auth(self._gateway_url, self.auth)

        self.archive_v1 = ArchiveV1(self._client, self._gateway_url)
        self.orders_v1 = OrdersV1(self._client, self._gateway_url)

        self.future = FutureApis(self._client, self._gateway_url)

    def _setup_client(self) -> AbstractClient:
        client = self._setup_requests_session()
        if client is None:
            client = UrllibClient()
        return client

    def _setup_requests_session(self) -> Union[AbstractClient, None]:
        try:
            from requests import Session

            from satellitevu.http.requests import RequestsSession
        except ImportError:
            return

        # TODO: Retries
        # TODO: Timeout
        session = Session()
        client = RequestsSession(instance=session)
        return client



from configparser import ConfigParser
from pathlib import Path

from pyfakefs.fake_filesystem import FakeFilesystem

from .cache import AppDirCache, MemoryCache

TEST_DIR = Path("/test")


def test_empty_cache_save(fs: FakeFilesystem):
    fs.makedir(TEST_DIR)
    cache = AppDirCache(TEST_DIR)
    parser = ConfigParser()

    cache.save("test-client", "test-access")

    parser.read(TEST_DIR / "tokencache")
    assert {key: value for key, value in parser.items("test-client")} == {
        "access_token": "test-access",
    }


def test_empty_cache_load(fs: FakeFilesystem):
    fs.makedir(TEST_DIR)
    cache = AppDirCache(TEST_DIR)

    assert cache.load("test-client") is None


def test_partial_cache_save(fs: FakeFilesystem):
    fs.makedir(TEST_DIR)
    cache = AppDirCache(TEST_DIR)
    cache_file = TEST_DIR / "tokencache"

    parser = ConfigParser()
    parser.add_section("other-client")
    parser["other-client"]["access_token"] = "foo"
    with open(cache_file, "w") as handle:
        parser.write(handle)

    cache.save("test-client", "test-access")

    parser = ConfigParser()
    parser.read(cache_file)
    assert {key: value for key, value in parser.items("test-client")} == {
        "access_token": "test-access",
    }
    assert {key: value for key, value in parser.items("other-client")} == {
        "access_token": "foo",
    }


def test_cache_update(fs: FakeFilesystem):
    fs.makedir(TEST_DIR)
    cache = AppDirCache(TEST_DIR)
    cache_file = TEST_DIR / "tokencache"

    parser = ConfigParser()
    parser.add_section("test-client")
    parser["test-client"]["access_token"] = "foo"
    with open(cache_file, "w") as handle:
        parser.write(handle)

    cache.save("test-client", "bar")

    parser = ConfigParser()
    parser.read(cache_file)
    assert {key: value for key, value in parser.items("test-client")} == {
        "access_token": "bar",
    }


def test_memory_cache_empty():
    cache = MemoryCache()

    assert cache.load("test-client") is None


def test_memory_cache():
    cache = MemoryCache()
    cache.save("test-client", "bar")

    assert cache.load("test-client") == "bar"



from importlib import import_module
from json import dumps
from typing import Dict, Optional
from urllib.parse import urljoin

from mocket.mockhttp import Entry
from pytest import fixture, mark, param

try:
    from requests import Session as RequestsSession
except ImportError:
    RequestsSession = None
try:
    from httpx import Client as HttpxClient
except ImportError:
    HttpxClient = None

from satellitevu.auth.cache import AbstractCache
from satellitevu.client import Client


@fixture(
    params=(
        param("UrllibClient"),
        param(
            "requests.RequestsSession",
            marks=[
                mark.skipif(RequestsSession is None, reason="requests is not installed")
            ],
        ),
        param(
            "httpx.HttpxClient",
            marks=[mark.skipif(HttpxClient is None, reason="httpx is not installed")],
        ),
    )
)
def http_client_class(request):
    full_path = f"satellitevu.http.{request.param}"
    module = import_module(full_path.rsplit(".", maxsplit=1)[0])
    return getattr(module, full_path.rsplit(".")[-1])


class MemoryCache(AbstractCache):
    cache: Dict[str, str]

    def __init__(self):
        super().__init__()
        self.cache = {}

    def save(self, client_id: str, value: str):
        self.cache[client_id] = value

    def load(self, client_id: str) -> Optional[str]:
        return self.cache.get(client_id)


@fixture
def client(memory_cache):
    return Client(client_id="mock-id", client_secret="mock-secret", cache=memory_cache)


@fixture
def memory_cache():
    return MemoryCache()


@fixture
def oauth_token_entry(client):
    return Entry.single_register(
        "POST",
        urljoin(client.auth.auth_url, "oauth/token"),
        body=dumps({"access_token": "mock-token"}),
    )


@fixture
def redirect_response():
    return {"url": "https://image.test"}


@fixture
def order_details_response():
    return {
        "id": "528b0f77-5df1-4ed7-9224-502817170613",
        "type": "FeatureCollection",
        "features": [
            {
                "id": "8566d747-8833-44a7-a1c8-8d13e541f673",
                "properties": {
                    "item_id": "image",
                    "owned_by": "random",
                    "created_at": "2022-10-20T09:23:14.959823",
                    "updated_at": "2022-10-20T09:23:14.959823",
                },
            }
        ],
    }



from json import dumps
from typing import List
from urllib.parse import parse_qs

from mocket import Mocket, Mocketizer
from mocket.mockhttp import Entry, Request
from pytest import mark

from satellitevu.apis.base import AbstractApi
from satellitevu.auth import Auth
from satellitevu.http.base import AbstractClient


class TestApi(AbstractApi):
    __test__ = False

    api_path = "/test"
    scopes = ["test"]


@mark.parametrize("kwargs", ({}, {"scopes": ["foo"]}))
def test_scopes(kwargs, http_client_class, memory_cache):
    client: AbstractClient = http_client_class()
    auth = Auth(
        client_id="test",
        client_secret="test",
        auth_url="http://auth.example.com",
        cache=memory_cache,
    )
    client.set_auth("http://api.example.com", auth)
    api = TestApi(client, "http://api.example.com")

    Entry.single_register(
        "POST",
        "http://auth.example.com/oauth/token",
        body=dumps({"access_token": "mock-token"}),
    )
    Entry.single_register("GET", api.url("/"), body="")
    with Mocketizer():
        api.make_request("GET", api.url("/"), **kwargs)
        requests: List[Request] = Mocket.request_list()

    assert len(requests) == 2
    assert parse_qs(requests[0].body)["scope"] == kwargs.get("scopes", api.scopes)



import tempfile
from io import BytesIO
from json import dumps
from unittest.mock import patch
from urllib.parse import urljoin, urlparse

import pytest
from mocket import Mocket, mocketize
from mocket.mockhttp import Entry
from pytest import mark

from satellitevu.apis.orders import bytes_to_file
from satellitevu.auth.exc import Api401Error, Api403Error


@mocketize(strict_mode=True)
@mark.parametrize(
    "item_ids",
    (
        "20221005T214049000_basic_0_TABI",
        "20220923T222227000_basic_0_TABI",
    ),
)
def test_submit_single_item(client, oauth_token_entry, item_ids):
    payload = dumps({"item_id": [item_ids]})

    Entry.single_register("POST", client._gateway_url + "orders/v1/", body=payload)

    response = client.orders_v1.submit(item_ids)

    requests = Mocket.request_list()
    assert len(requests) == 2

    api_request = requests[-1]
    assert api_request.headers["Host"] == urlparse(client._gateway_url).hostname
    assert api_request.path == "/orders/v1/"
    assert api_request.headers["Content-Type"] == "application/json"
    assert api_request.headers["Authorization"] == "Bearer mock-token"
    assert api_request.body == payload

    assert response.status == 200


@mocketize(strict_mode=True)
@mark.parametrize(
    "item_ids",
    (
        ["20221005T214049000_basic_0_TABI", "20220923T222227000_basic_0_TABI"],
        ["20220923T222227000_basic_0_TABI"],
    ),
)
def test_submit_multiple_items(client, oauth_token_entry, item_ids):
    payload = dumps({"item_id": item_ids})

    Entry.single_register("POST", client._gateway_url + "orders/v1/", body=payload)

    response = client.orders_v1.submit(item_ids)

    requests = Mocket.request_list()
    assert len(requests) == 2

    api_request = requests[-1]
    assert api_request.headers["Host"] == urlparse(client._gateway_url).hostname
    assert api_request.path == "/orders/v1/"
    assert api_request.headers["Content-Type"] == "application/json"
    assert api_request.headers["Authorization"] == "Bearer mock-token"
    assert api_request.body == payload

    assert response.status == 200


@mocketize(strict_mode=True)
@mark.parametrize(
    "item_ids, status, exception",
    (
        ("20221005T214049000_basic_0_TABI", 401, Api401Error),
        (
            ["20221005T214049000_basic_0_TABI", "20220923T222227000_basic_0_TABI"],
            403,
            Api403Error,
        ),
    ),
)
def test_cannot_submit_unauthorized_order(
    client, oauth_token_entry, item_ids, status, exception
):
    if isinstance(item_ids, str):
        item_ids = [item_ids]
    payload = dumps({"item_id": item_ids})

    Entry.single_register(
        "POST", client._gateway_url + "orders/v1/", body=payload, status=status
    )

    with pytest.raises(exception):
        client.orders_v1.submit(item_ids)

    requests = Mocket.request_list()
    assert len(requests) == 2

    api_request = requests[-1]
    assert api_request.headers["Host"] == urlparse(client._gateway_url).hostname
    assert api_request.path == "/orders/v1/"
    assert api_request.headers["Content-Type"] == "application/json"
    assert api_request.headers["Authorization"] == "Bearer mock-token"
    assert api_request.body == payload


@mocketize(strict_mode=True)
def test_item_download_url(client, oauth_token_entry, redirect_response):
    order_id = "uuid"
    item_id = "image"

    Entry.single_register(
        "GET",
        client._gateway_url + f"orders/v1/{order_id}/{item_id}/download?redirect=False",
        body=dumps(redirect_response),
    )

    response = client.orders_v1.item_download_url(order_id, item_id)
    requests = Mocket.request_list()

    assert len(requests) == 2
    api_request = requests[-1]
    assert api_request.headers["Host"] == urlparse(client._gateway_url).hostname
    assert api_request.path == "/orders/v1/uuid/image/download?redirect=False"
    assert api_request.headers["Authorization"] == "Bearer mock-token"

    assert isinstance(response, dict)
    assert response["url"] == "https://image.test"


@mocketize(strict_mode=True)
@mark.parametrize("status, exception", ((401, Api401Error), (403, Api403Error)))
def test_no_access_to_download_if_unauthorized(
    client, oauth_token_entry, status, exception, redirect_response
):
    order_id = "uuid"
    item_id = "image"

    Entry.single_register(
        "GET",
        client._gateway_url + f"orders/v1/{order_id}/{item_id}/download?redirect=False",
        body=dumps(redirect_response),
        status=status,
    )

    with pytest.raises(exception):
        client.orders_v1.item_download_url(order_id, item_id)

    requests = Mocket.request_list()

    assert len(requests) == 2
    api_request = requests[-1]
    assert api_request.headers["Host"] == urlparse(client._gateway_url).hostname
    assert api_request.path == "/orders/v1/uuid/image/download?redirect=False"
    assert api_request.headers["Authorization"] == "Bearer mock-token"


@mocketize(strict_mode=True)
def test_download_order_item(client, oauth_token_entry, redirect_response):
    order_id = "uuid"
    item_id = "image"

    Entry.single_register(
        "GET",
        client._gateway_url + f"orders/v1/{order_id}/{item_id}/download?redirect=False",
        body=dumps(redirect_response),
    )

    Entry.single_register("GET", uri="https://image.test")

    requests = Mocket.request_list()

    with patch("satellitevu.apis.orders.bytes_to_file") as mock_file_dl:
        mock_file_dl.return_value = "Downloads/image.zip"
        response = client.orders_v1.download_item(order_id, item_id, "Downloads")

    assert len(requests) == 3

    api_request = requests[1]
    assert api_request.headers["Host"] == urlparse(client._gateway_url).hostname
    assert api_request.path == "/orders/v1/uuid/image/download?redirect=False"
    assert api_request.headers["Authorization"] == "Bearer mock-token"

    mock_file_dl.assert_called_once()
    assert response == mock_file_dl()

    Mocket.assert_fail_if_entries_not_served()


@mocketize(strict_mode=True)
def test_get_order_details(client, oauth_token_entry, order_details_response):
    fake_uuid = "528b0f77-5df1-4ed7-9224-502817170613"

    Entry.single_register(
        "GET",
        client._gateway_url + f"orders/v1/{fake_uuid}",
        body=dumps(order_details_response),
    )

    response = client.orders_v1.get_order_details(fake_uuid)

    requests = Mocket.request_list()
    assert len(requests) == 2

    api_request = requests[-1]
    assert api_request.headers["Host"] == urlparse(client._gateway_url).hostname
    assert api_request.path == f"/orders/v1/{fake_uuid}"
    assert api_request.headers["Authorization"] == "Bearer mock-token"

    assert isinstance(response, dict)


@mocketize(strict_mode=True)
@mark.parametrize("status, exception", ((401, Api401Error), (403, Api403Error)))
def test_cannot_get_order_details_if_unauthorized(
    client, oauth_token_entry, status, exception, order_details_response
):
    fake_uuid = "528b0f77-5df1-4ed7-9224-502817170613"

    Entry.single_register(
        "GET",
        client._gateway_url + f"orders/v1/{fake_uuid}",
        body=dumps(order_details_response),
        status=status,
    )

    with pytest.raises(exception):
        client.orders_v1.get_order_details(fake_uuid)

    requests = Mocket.request_list()
    assert len(requests) == 2

    api_request = requests[-1]
    assert api_request.headers["Host"] == urlparse(client._gateway_url).hostname
    assert api_request.path == f"/orders/v1/{fake_uuid}"
    assert api_request.headers["Authorization"] == "Bearer mock-token"


@mocketize(strict_mode=True)
def test_download_order(client, order_details_response, redirect_response):
    fake_uuid = "528b0f77-5df1-4ed7-9224-502817170613"
    download_dir = "downloads"

    Entry.single_register(
        "POST",
        urljoin(client.auth.auth_url, "oauth/token"),
        body=dumps({"access_token": None}),
    )

    Entry.single_register(
        "GET",
        client._gateway_url + f"orders/v1/{fake_uuid}",
        body=dumps({**order_details_response, **{"access_token": None}}),
    )

    with patch("satellitevu.apis.orders.OrdersV1._save_order_to_zip") as mock_zip:
        mock_zip.return_value = f"{download_dir}/SatelliteVu_{fake_uuid}.zip"
        response = client.orders_v1.download_order(fake_uuid, download_dir)

    requests = Mocket.request_list()

    assert len(requests) == 2

    api_request = requests[1]
    assert api_request.headers["Host"] == urlparse(client._gateway_url).hostname
    assert api_request.path == f"/orders/v1/{fake_uuid}"
    assert api_request.headers["Authorization"] == "Bearer None"

    mock_zip.assert_called_once()

    assert isinstance(response, str)
    assert response == mock_zip()

    Mocket.assert_fail_if_entries_not_served()


@mocketize(strict_mode=True)
@mark.parametrize("status, exception", ((401, Api401Error), (403, Api403Error)))
def test_download_order_unauthorized(
    client, order_details_response, redirect_response, status, exception
):
    fake_uuid = "528b0f77-5df1-4ed7-9224-502817170613"
    download_dir = "downloads"

    Entry.single_register(
        "POST",
        urljoin(client.auth.auth_url, "oauth/token"),
        body=dumps({"access_token": None}),
    )

    Entry.single_register(
        "GET",
        client._gateway_url + f"orders/v1/{fake_uuid}",
        body=dumps({**order_details_response, **{"access_token": None}}),
        status=status,
    )

    with pytest.raises(exception):
        client.orders_v1.download_order(fake_uuid, download_dir)

    requests = Mocket.request_list()

    assert len(requests) == 2

    api_request = requests[1]
    assert api_request.headers["Host"] == urlparse(client._gateway_url).hostname
    assert api_request.path == f"/orders/v1/{fake_uuid}"
    assert api_request.headers["Authorization"] == "Bearer None"


def test_bytes_to_file():
    outfile_path = tempfile.mkstemp()[1]
    output = bytes_to_file(BytesIO(b"Hello world"), outfile_path)

    assert isinstance(output, str)
